/****
 Stub for invoking sqlj Command-line tool
 ****/

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <process.h>
#include <string.h>
#include <errno.h>
#include <io.h>

#define ENV_VAR     "SQLJ_OPTIONS="     /* environment variable with SQLJ settings */
#define JAVA        "java"              /* invocation of Java interpreter */
#define sqlj_MAIN   "sqlj.tools.Sqlj"   /* main class for sqlj tool */

#define tempname    "tempfile"          /* name for temporary file */

/************************************/
/*  Is this a flag for the Java VM? */
/*  Return the actual flag if yes   */
/************************************/
static int classpathSeen = 0;

char * getJavaFlag(char * str)
{
   if (classpathSeen)
   {
     classpathSeen = 0;
     return str;
   };

   if (strcmp(str,"-J-classpath")==0)
   {
     classpathSeen = 1;
   }

   if (strncmp(str, "-J-", 3)==0)
   {
     return (&str[2]);
   }

   return NULL;
}


/*****************************************/
/*  Is this the flag to set the JavaVM?  */
/*  Return it if yes.                    */
/*****************************************/

char * getJavaVMFlag(char * str)
{
   if (strncmp(str, "-vm=", 4)==0)
   {
     return (&str[4]);
   }

   return NULL;
}


/*****************************************/
/*  Is this the flag to set the JavaVM?  */
/*  Return it if yes.                    */
/*****************************************/

int isPassesFlag(char * str)
{
   if (strcmp(str, "-passes")==0)
   {
 	  return 1;
   }

   return 0;
}

/******************************************/
/*  Report stub errors                    */
/*  Return true if an error was reported  */
/******************************************/

int reportStubError(int i, char * sqlj_prog)
{
  if (i==-1)
  {
    if (errno==E2BIG)
    {
      printf("Error in sqlj stub: argument list too long.");
    }
    else if (errno==EINVAL)
    {
      printf("Error in sqlj stub: invalid argument.");
    }
    else if (errno==ENOENT)
    {
     printf("Error in sqlj stub: path or file name \"%s\" not found.", sqlj_prog);
    }
    else if (errno==ENOEXEC)
    {
     printf("Error in sqlj stub: exec format error.");
    }
    else if (errno==ENOMEM)
    {
     printf("Error in sqlj stub: not enough memory.");
    }
	else  /* some executables want to fool us and return -1    */
		  /* when _they_ have a problem.  We detect them here! */
	{
	  return 0;
	}
	return i;
  }

  return 0;
}


/**************************/
/*  Program Main Routine  */
/**************************/


int main (int argc, char *argv[], char *envp[])
{
  char *sqlj_prog = JAVA;

  char ** new_argv;

  int i, l;
  int idx;
  int envargscount = 0;
  char ** envargs;
  char *envvalue = NULL;

  int hasPasses = 0;
  char passArgument[255];

  
  /*************************************/
  /*****  GET ENVIRONMENT SETTING  *****/
  /*************************************/

  /***  Check for environment variable  ***/
  
  while( *envp != NULL )
  {
	 if (strncmp(*envp, ENV_VAR, strlen(ENV_VAR))==0)
     {
		envvalue = &(*envp)[ strlen(ENV_VAR)]; 
		break;
	 }
	 envp++;
  }

  
  /*** Retrieve options from environment variable. ***/

  if (envvalue!=NULL)
  {
	 l = strlen(envvalue);

	 /* Guess the number of arguments */
	 envargscount=1;      /* at least one argument + number of whitespace chars */
	 for (i=0; i<l; i++)
	 {
	   if (envvalue[i]==' ' || envvalue[i]=='\t')
	   {
		  envargscount++;
	   }
	 }

     /* Allocate array for arguments */
     envargs = calloc(envargscount, sizeof (char *));
     if (envargs==NULL)
	 {
       printf("Error in sqlj stub: not enough memory.");
       return -1;
	 }
     
	 /* Populate argument array */
	 envargscount=0;
	 i = 0;
     while (envvalue != NULL)
	 {
       int j = 0;
       int done = 0;

	   while (envvalue[i]==' ' || envvalue[i]=='\t') i++;
	 
	   j=i;
	   while (envvalue[i]!=' ' && envvalue[i]!='\t' && envvalue[i]!='\0') i++;
	   done = (envvalue[i]=='\0');

	   if (i!=j)
	   {
		 envvalue[i]='\0';
         envargs[envargscount] = &(envvalue[j]);
		 envargscount++;
	   }
	   if (done)
	   {
		  break;
	   }
	   else
	   {
		 i++;
	   }
	 }
  }

  
  /**************************************/
  /*****  CONSTRUCT ARGUMENT ARRAY  *****/
  /**************************************/

  new_argv = calloc(3 + envargscount + argc, sizeof (char *));
  if (new_argv==NULL)
  {
    printf("Error in sqlj stub: not enough memory.");
    return -1;
  }


  /***  Add JavaVM invocation  ***/

  idx = 0;
  new_argv[idx] = sqlj_prog; idx++;


  /*** Add Flags to JavaVM ***/

  for (i = 1; i < envargscount + argc; i++)
  {
    char * javaArg;
    char * argi = (i<=envargscount) ? envargs[i-1]
		                            : argv[i - envargscount];

    new_argv[i] = NULL;

	if ((javaArg = getJavaFlag(argi)))
    {
      new_argv[idx] = javaArg;
      idx++;
    }
	else if (isPassesFlag(argi))
    {
	   hasPasses = 1;
	}
  }
  new_argv[envargscount + argc]     = NULL;
  new_argv[envargscount + argc + 1] = NULL;
  new_argv[envargscount + argc + 2] = NULL;


  /***  Add Class invoked by JavaVM  ***/
  
  new_argv[idx] = sqlj_MAIN;
  idx++;


  /*** Add the first pass flag to SQLJ - if required ***/

  if (hasPasses)
  {
    strcpy(passArgument,"-pass1=");
	strcat(passArgument, tempname);
    new_argv[idx] = passArgument;
	hasPasses = idx;                   /* remember where the pass argument lives */
	idx++;
  }

  
  /***  Add remaining SQLJ arguments to runline  ***/

  for (i = 1; i< envargscount + argc; i++)
  { 
    char * javavm;
    char * argi = (i<=envargscount) ? envargs[i-1]
		                            : argv[i - envargscount];

    if (getJavaFlag(argi) || isPassesFlag(argi))
    {
      /* skip it */
    }
    else if ((javavm = getJavaVMFlag(argi)))
    {
      new_argv[0] = javavm;
      sqlj_prog = javavm;
    }
    else
    {
      new_argv[idx] = argi; idx++;
    }
  }

  
  /*********************************/
  /*****  EXECUTE THE RUNLINE  *****/
  /*********************************/

  i = spawnvp(P_WAIT, sqlj_prog, new_argv);

  if (i && reportStubError(i, sqlj_prog)) exit(i);


  /******************************************************/
  /*****  EXECUTE THE JAVA COMPILER - IF REQUESTED  *****/
  /******************************************************/

  if (hasPasses)
  {
	 FILE *stream = NULL;
	 long pos;
     char * rawArgs;
     char runfile[255];
	 
	 /***  try opening the file with the runline information  ***/

	 strcpy(runfile, tempname);
	 strcat(runfile, ".run");
	 stream = fopen(runfile, "rb");

 
	 /***  read runline ***/
	 
	 if (stream != NULL)
	 {
		int done = 0;

		/* figure out the size of this file */
		
		fseek(stream, 0, SEEK_END);
		pos = ftell(stream);
        fseek(stream, 0, SEEK_SET);

		
		rawArgs = malloc(pos + 1 + strlen(tempname) + strlen("> .log "));
		
		if (rawArgs==NULL)
		{
          printf("Error in sqlj stub: not enough memory.");
          exit(-1);
		};
		fread(rawArgs, pos, 1, stream);
		fclose(stream);

		/* terminate argument string */
		rawArgs[pos] = '\0';
		
		pos--;  /* remove trailing CR and LF characters */
		while (0<= pos && rawArgs[pos]=='\r' || rawArgs[pos]=='\n') 
		{
		  rawArgs[pos]='\0';
		  pos--;
		}

		/* put together argument string */
		strcat(rawArgs, " >");
		strcat(rawArgs, tempname);
		strcat(rawArgs, ".log");
        
		fflush(stdout);

		/* execute the Jav compiler */
		i = system(rawArgs);
        if (i && reportStubError(i, sqlj_prog)) exit(i);
	 }
  }


  /**********************************************************/
  /*****  EXECUTE THE SECOND SQLJ PHASE - IF REQUESTED  *****/
  /**********************************************************/

  if (hasPasses)
  {
	 char * pflag = (i) ? "-fail2" : "-pass2";
	 int j;

	 for (j=0; j<6; j++)
	 {
	   new_argv[hasPasses][j] = pflag[j];
	 }

	 i = spawnvp(P_WAIT, sqlj_prog, new_argv);
     
	 if (i && reportStubError(i, sqlj_prog)) exit(i);
  }

  return i;
}
