/* Copyright (c) 1997 Oracle Corporation */
import java.sql.*;
import sqlj.runtime.ref.DefaultContext ;

/**
  This is a utility class for establishing SQLJ connections.
  **/
public class ConnectionManager {

  private static String uid    = "scott" ;
  private static String pwd    = "tiger" ;
  
  private static String dburl  = "jdbc:oracle:oci8:@" ; // for OCI 8
  // private static String dburl  = "jdbc:oracle:oci7:@" ; // for OCI 7
  // private static String dburl  = "jdbc:oracle:thin:@localhost:1521:ORCL"; // for thin

  private static String driver = "oracle.jdbc.driver.OracleDriver";
  
  /** 
    Creates and returns a new JDBC Connection object. If an error occurs, 
    an error message is printed to System.err and the programs exits.
    **/ 
  static public Connection newConnection()  {
    Connection conn = null;
    
    //Verify that the access parameters are defined.
    if (uid== null || pwd==null || dburl==null )   {
      System.err.println ("Please set DBURL, UID, and PWD, and try again." ) ;
      System.exit(1) ;
    }
    
    try {
      Class driverClass = Class.forName(driver);
      DriverManager.registerDriver((Driver)driverClass.newInstance());
      conn = DriverManager.getConnection (dburl, uid, pwd);
    } catch (Exception e) {
      System.err.println( "Could not establish the JDBC connection:") ;
      System.err.println(e);  System.exit(1); 
    }
    return conn;
  }
  
  /** 
    Returns the currently installed SQLJ default context.  If the current
    default context is null, a new default context instance is set up using 
    a connection obtained from <code>getConnection()</code>.  
    **/ 
  static public DefaultContext initContext()  
  {
    DefaultContext ctx = DefaultContext.getDefaultContext();
    if (ctx == null) {
      try {
	ctx = new DefaultContext(newConnection());
      } catch (SQLException e) {
	System.out.println("Error: could not get a default context");
	System.err.println(e) ;  System.exit(1);
      }
      DefaultContext.setDefaultContext(ctx);
    }
    return ctx;
  }
}
